from sqlobject import *
from extracols import DictPickleCol
import time

from anonylink import AnonymityLink

class FormDataEvolution(SQLObject):
    _cacheValues = False

    who = ForeignKey('User', default = None)
    status = StringCol(length = 40)
    time = DateTimeCol()
    comment = StringCol(default = None)
    formdata = ForeignKey('FormData')

    #def __init__(self, status, comment, who):
    #   self.time = time.localtime() #XXX: do not forget

class FormData(SQLObject):
    key = 'id'
    names = 'forms'
    _cacheValues = False

    user = ForeignKey('User', default = None)
    receipt_time = DateTimeCol(default = None)
    status = StringCol(default = None, length = 40)
    evolution = MultipleJoin('FormDataEvolution', joinMethodName = 'evolution',
            joinColumn = 'formdata_id')
    data = DictPickleCol(default = None)

    formdef = ForeignKey('FormDef', default = None)
    consultation = ForeignKey('Consultation', default = None)


    def migrateOldData(cls):
        import storage
        from formdef import FormDef
        from consultationdef import Consultation
        from users import User

        def migrate(objectdef):
            if isinstance(objectdef, FormDef):
                formdataname = 'formtmp-%s' % objectdef.id
            elif isinstance(objectdef, Consultation):
                formdataname = 'consultationtmp-%s' % objectdef.id
            formdata_init_method = FormData.__init__
            FormData.__init__ = lambda x: x
            formdatadicts = [x.__dict__ for x in storage.get_storage().values(formdataname)]
            FormData.__init__ = formdata_init_method
            fields = objectdef.fields
            for formdict in formdatadicts:
                formdata = FormData()
                if isinstance(objectdef, FormDef):
                    formdata.formdef = objectdef
                elif isinstance(objectdef, Consultation):
                    formdata.consultation = objectdef
                if formdict.get('user_id'):
                    user_id = formdict.get('user_id')
                    if user_id == 'ultra-user':
                        pass # ignore, this shouldn't have happened
                    elif str(user_id).startswith('anonymous-'):
                        a = AnonymityLink(formdata = formdata)
                        a.name_identifier = user_id[10:]
                    elif str(user_id).startswith('_'): # really old wcs versions
                        a = AnonymityLink(formdata = formdata)
                        a.name_identifier = user_id
                    else:
                        try:
                            formdata.user = User.get(user_id)
                        except ValueError:
                            a = AnonymityLink(formdata = formdata)
                            a.key = user_id

                formdata.status = formdict.get('status')
                formdata.receipt_time = formdict.get('receipt_time')
                newdata = {}
                for k, v in formdict.get('data', {}).items():
                    # changed keys from 'field name' to 'field id'
                    try:
                        field = [x for x in fields if x.label == k][0]
                    except IndexError:
                        continue
                    newdata[field.id] = v
                formdata.data = newdata
                for evo in formdict.get('evolution', []):
                    evo_dict = evo.__dict__
                    evo = FormDataEvolution(formdata = formdata,
                            time = evo_dict.get('time'),
                            status = evo_dict.get('status'))
                    if evo_dict.get('who'):
                        evo.who = User.get(evo_dict.get('who'))
                    evo.comment = evo_dict.get('comment')

        for objectdef in FormDef.select():
            migrate(objectdef)
        for objectdef in Consultation.select():
            migrate(objectdef)
    migrateOldData = classmethod(migrateOldData)

class Evolution: # for backward compatibility with old pickled files
    pass
