from sqlobject import *

def get_logged_users_role():
    return Role.select(AND(Role.q.name == 'Logged Users', Role.q.system == True))[0]

class RoleEmail(SQLObject):
    email = StringCol(length = 100)
    role = ForeignKey('Role')
    _cacheValues = False


class Role(SQLObject):
    key = 'id'
    names = 'roles'
    _cacheValues = False

    name = StringCol(length = 100)
    details = StringCol(default = None)
    emails = MultipleJoin('RoleEmail', joinMethodName = 'emails')
    system = BoolCol(default = False)

    users = RelatedJoin('User')
    formdefs = RelatedJoin('FormDef')
    consultations = RelatedJoin('Consultation')

    def initTable(cls):
        role = Role(name = N_('Logged Users'))
        role.details = 'System Group for Logged Users'
        role.system = True
    initTable = classmethod(initTable)

    def migrateOldData(cls):
        import storage
        from users import User
        from formdef import FormDef
        from consultationdef import Consultation

        init_method = cls.__init__
        cls.__init__ = lambda x: x
        roles = storage.get_storage().values('roles')
        items = [x.__dict__ for x in roles]
        cls.__init__ = init_method

        new_ids_mapping = [('site-admin', 'site-admin')]
        for item in items:
            role = Role(name = item['name'])
            role.details = item.get('details')
            for email in item.get('emails', []):
                RoleEmail(email = email, role = role)
            new_ids_mapping.append( (item['id'], role.id) )

        ## fixing other tables to get the new ids
        # users
        user_init_method = User.__init__
        User.__init__ = lambda x: x
        users = storage.get_storage().values('users')
        for user in users:
            current_roles = user.__dict__['roles'][:]
            user.__dict__['roles'] = []
            for role in current_roles:
                if not role in [x[0] for x in new_ids_mapping]:
                    continue # unknown role, skipping
                user.__dict__['roles'].append([x[1] for x in new_ids_mapping if x[0] == role][0])
            storage.get_storage().store(user)
        User.__init__ = user_init_method

        # formdefs
        formdef_init_method = FormDef.__init__
        FormDef.__init__ = lambda x: x
        formdefs = storage.get_storage().values('formdefs')
        for formdef in formdefs:
            role = formdef.__dict__['receiver']
            if not role in [x[0] for x in new_ids_mapping]:
                formdef.__dict__['receiver'] = -1
            else:
                formdef.__dict__['receiver'] = [x[1] for x in new_ids_mapping if x[0] == role][0]

            current_roles = formdef.__dict__.get('roles', [])[:]
            formdef.__dict__['roles'] = []
            for role in current_roles:
                if not role in [x[0] for x in new_ids_mapping]:
                    continue # unknown role, skipping
                formdef.__dict__['roles'].append([x[1] for x in new_ids_mapping if x[0] == role][0])
            storage.get_storage().store(formdef)
        FormDef.__init__ = formdef_init_method

        # consultations
        consultation_init_method = Consultation.__init__
        Consultation.__init__ = lambda x: x
        consultations = storage.get_storage().values('consultations')
        for consultation in consultations:
            role = consultation.__dict__['receiver']
            consultation.__dict__['receiver'] = [x[1] for x in new_ids_mapping if x[0] == role][0]
            current_roles = formdef.__dict__.get('roles', [])[:]
            consultation.__dict__['roles'] = []
            for role in current_roles:
                if not role in [x[0] for x in new_ids_mapping]:
                    continue # unknown role, skipping
                consultation.__dict__['roles'].append([x[1] for x in new_ids_mapping \
                        if x[0] == role][0])
            storage.get_storage().store(consultation)
        Consultation.__init__ = consultation_init_method
    migrateOldData = classmethod(migrateOldData)

