from sqlobject import *

from quixote.session import Session, SessionManager
from quixote.util import randbytes

from extracols import DictPickleCol, ListPickleCol

class SqlSession(SQLObject):
    _cacheValue = False
    
    session_id = StringCol(notNone = True, alternateID = True, unique = True, length = 255)
    user = StringCol(default = None)
    remote_address = StringCol(default = None, length=30)
    creation_time = FloatCol(default = None)
    access_time = FloatCol(default = None)

    name_identifier = StringCol(length = 100, default = None)
    anonymous_key = StringCol(length = 100, default = None)

    after_url = StringCol(default = None)
    lasso_session_dump = StringCol(default = None)
    lasso_anonymous_identity_dump = StringCol(default = None)

    tempfiles = DictPickleCol(default = None)
    form_tokens = ListPickleCol(default = None)

    def _init(self, *args, **kw):
        return SQLObject._init(self, *args, **kw)


class BasicSession(Session):
    name_identifier = None
    after_url = None
    anonymous_key = None
    tempfiles = None
    lasso_session_dump = None
    lasso_anonymous_identity_dump = None
    tempfiles = None

    def has_info(self):
        return self.name_identifier or self.after_url or self.anonymous_key or \
            self.tempfiles or self.lasso_session_dump or \
            self.lasso_anonymous_identity_dump or Session.has_info(self)
    is_dirty = has_info

    def get_anonymous_key(self, generate = False):
        if self.anonymous_key:
            return self.anonymous_key
        if generate:
            self.anonymous_key = randbytes(20)
        return self.anonymous_key


db_attributes = ['user', 'name_identifier', 'anonymous_key', 'after_url', 'lasso_session_dump',
        'lasso_anonymous_identity_dump', 'tempfiles']
db_underscore_attributes = ['_remote_address', '_creation_time', '_access_time',
        '_form_tokens']

class SqlSessionMapping:
    def keys(self):
        return [x.session_id for x in SqlSession.select()]

    def values(self):
        return [self._from_db(x) for x in SqlSession.select()]

    def items(self):
        return [(x.session_id, self._from_db(x)) for x in SqlSession.select()]

    def _from_db(self, sqlsession):
        session = BasicSession(sqlsession.session_id)
        for attr in db_attributes:
            setattr(session, attr, getattr(sqlsession, attr))
        for attr in db_underscore_attributes:
            setattr(session, attr, getattr(sqlsession, attr[1:]))
        return session

    def _to_db(self, session):
        try:
            sqlsession = SqlSession.bySession_id(session.id)
        except SQLObjectNotFound:
            sqlsession = SqlSession(session_id = session.id)

        kw = {}
        for attr in db_attributes:
            kw[attr] = getattr(session, attr)
        for attr in db_underscore_attributes:
            kw[attr[1:]] = getattr(session, attr)
        sqlsession.set(**kw)

    def get(self, session_id, default = None):
        if session_id is None:
            return default
        try:
            return self._from_db(SqlSession.bySession_id(session_id))
        except SQLObjectNotFound:
            return default
    
    def __getitem__(self, k):
        return self.get(k)

    def has_key(self, k):
        return (self.get(k) is not None)

    def __setitem__(self, k, v):
        pass

    def __delitem__(self, k):
        SqlSession.bySession_id(k).destroySelf()
        

class SqlSessionManager(SessionManager):
    def __init__(self):
        SessionManager.__init__(self, BasicSession, SqlSessionMapping())

    def commit_changes(self, session):
        if session and session.has_info():
            self.sessions._to_db(session)
