import os
import cPickle

from quixote import get_publisher

class Storage:
    pass


class Storable:
    def get_table_name(self):
        if hasattr(self, 'names'):
            return self.names
        return self.__class__.__name__.lower()

def fixkey(k):
    # insure key can be inserted in filesystem
    if not k: return k
    return str(k).replace('/', '-')

class FilesStorage:
    def __init__(self, base_dir):
        self.base_dir = base_dir
        if self.base_dir and not os.path.exists(self.base_dir):
            os.mkdir(self.base_dir)

    def rename_table(self, obname, newname):
        dirname = os.path.join(self.base_dir, str(obname))
        newdirname = os.path.join(self.base_dir, str(newname))
        if os.path.exists(dirname):
            os.rename(dirname, newdirname)

    def keys(self, obname):
        dirname = os.path.join(self.base_dir, str(obname))
        if not os.path.exists(dirname):
            return []
        return [fixkey(x) for x in os.listdir(dirname)]

    def has_key(self, obname, key):
        return fixkey(key) in self.keys(obname)

    def retrieve(self, obname, key):
        if key is None:
            raise KeyError()
        dirname = os.path.join(self.base_dir, str(obname))
        try:
            return cPickle.load(open(os.path.join(dirname, fixkey(key))))
        except IOError:
            raise KeyError()
            
    def store(self, object):
        obname = object.names
        dirname = os.path.join(self.base_dir, obname)
        if not os.path.exists(dirname):
            os.mkdir(dirname)
        if not getattr(object, object.key):
            keys = self.keys(obname)
            key = len(keys) + 1
            while str(key) in keys:
                key += 1
            key = str(key)
            setattr(object, object.key, key)
        dirname = os.path.join(self.base_dir, str(obname))
        s = cPickle.dumps(object)
        file(os.path.join(dirname, fixkey(getattr(object, object.key))), 'w').write(s)

    def retrieve(self, obname, key):
        if key is None:
            raise KeyError()
        dirname = os.path.join(self.base_dir, str(obname))
        try:
            return cPickle.load(open(os.path.join(dirname, fixkey(key))))
        except (IOError, EOFError, cPickle.UnpicklingError):
            # XXX: logs error
            raise KeyError()

    def values(self, obname):
        return [self.retrieve(obname, k) for k in self.keys(str(obname))]

    def items(self, obname):
        return [(k, self.retrieve(obname, k)) for k in self.keys(str(obname))]

    def remove(self, object = None):
        obname = object.get_table_name()
        return self.remove_id(obname, fixkey(getattr(object, object.key)))

    def remove_id(self, obname, key):
        dirname = os.path.join(self.base_dir, str(obname))
        os.unlink(os.path.join(dirname, fixkey(key)))

    def remove_all(self, obname):
        dirname = os.path.join(self.base_dir, str(obname))
        if not os.path.exists(dirname):
            return
        for k in os.listdir(dirname):
            os.unlink(os.path.join(dirname, k))
        os.rmdir(dirname)
        

def get_storage():
    return FilesStorage(get_publisher().app_dir)

