from sqlobject import *

class NameIdentifier(SQLObject):
    name_identifier = StringCol()
    user = ForeignKey('User')
    _cacheValues = False


class User(SQLObject):
    key = 'id'
    names = 'users'
    _cacheValues = False

    name = StringCol(default = None, length = 100)
    email = StringCol(default = None, length = 100)
    roles = RelatedJoin('Role')
    name_identifiers = MultipleJoin('NameIdentifier', joinMethodName = 'name_identifiers')
    identification_token = StringCol(default = None)
    lasso_dump = StringCol(default = None)
    is_admin = BoolCol(default = False)

    def migrateOldData(cls):
        import storage
        from roles import Role
        from formdef import FormDef
        from formdata import FormData
        from consultationdef import Consultation
        init_method = cls.__init__
        cls.__init__ = lambda x: x
        users = storage.get_storage().values('users')
        items = [x.__dict__ for x in users]
        cls.__init__ = init_method

        items.sort(lambda x,y: cmp(x['id'], y['id']))
        new_ids_mapping = []
        for item in items:
            user = User(name = item['name'])
            user.email = item.get('email')
            user.identification_token = item.get('identification_token')
            user.lasso_dump = item.get('lasso_dump')
            if 'site-admin' in item.get('roles', []):
                user.is_admin = True
            for ni in item.get('name_identifiers', []):
                NameIdentifier(name_identifier = ni, user = user)
            for role in item.get('roles', []):
                if role == 'site-admin':
                    continue
                user.addRole(Role.get(role))
            new_ids_mapping.append( (item['id'], user.id) )

        ## Fixing other tables to get the new id
        formdef_init_method = FormDef.__init__
        FormDef.__init__ = lambda x: x
        consultation_init_method = Consultation.__init__
        Consultation.__init__ = lambda x: x
        formdata_init_method = FormData.__init__
        FormData.__init__ = lambda x: x
        formdefs = storage.get_storage().values('formdefs')
        consultations = storage.get_storage().values('consultations')
        for elem in formdefs + consultations:
            if isinstance(elem, FormDef):
                formdataname = 'form-%s' % elem.__dict__['id']
            elif isinstance(elem, Consultation):
                formdataname = 'consultation-%s' % elem.__dict__['id']
            else:
                raise "unknown"
            for formdata in storage.get_storage().values(formdataname):
                if hasattr(formdata, 'user_id') and formdata.user_id:
                    try:
                        formdata.user_id = [x[1] for x in new_ids_mapping \
                                           if x[0] == formdata.user_id][0]
                    except IndexError:
                        pass
                if formdata.__dict__.has_key('evolution'):
                    for evo in formdata.__dict__['evolution']:
                        if not hasattr(evo, 'who') or not evo.who:
                            continue
                        evo.who = [x[1] for x in new_ids_mapping if x[0] == evo.who][0]
                storage.get_storage().store(formdata)

        FormData.__init__ = formdata_init_method
        FormDef.__init__ = formdef_init_method
        Consultation.__init__ = consultation_init_method

    migrateOldData = classmethod(migrateOldData)

